'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _domLib = require('dom-lib');

var _ClassNameMixin = require('./mixins/ClassNameMixin');

var _ClassNameMixin2 = _interopRequireDefault(_ClassNameMixin);

var _ReactComponentWithPureRenderMixin = require('./mixins/ReactComponentWithPureRenderMixin');

var _ReactComponentWithPureRenderMixin2 = _interopRequireDefault(_ReactComponentWithPureRenderMixin);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function clamp(value, min, max) {
  if (value < min) {
    return min;
  }
  if (value > max) {
    return max;
  }
  return value;
}

var ColumnResizeHandler = _react2.default.createClass({
  displayName: 'ColumnResizeHandler',

  mixins: [_ClassNameMixin2.default, _ReactComponentWithPureRenderMixin2.default],
  propTypes: {
    height: _react.PropTypes.number,
    columnWidth: _react.PropTypes.number,
    columnLeft: _react.PropTypes.number,
    columnFixed: _react.PropTypes.bool,
    onColumnResizeStart: _react.PropTypes.func,
    onColumnResizeEnd: _react.PropTypes.func,
    onColumnResizeMove: _react.PropTypes.func
  },
  _columnWidth: 0,
  _cursorDelta: 0,
  _onMove: function _onMove(deltaX, deltaY) {

    if (!this.isKeyDown) {
      return;
    }

    this._cursorDelta += deltaX;
    this._columnWidth = clamp(this.props.columnWidth + this._cursorDelta, 20);
    this.props.onColumnResizeMove(this._columnWidth, this.props.columnLeft, this.props.columnFixed);
  },
  _onColumnResizeEnd: function _onColumnResizeEnd() {

    this.isKeyDown = false;

    this.props.onColumnResizeEnd(this._columnWidth, this._cursorDelta);

    if (this._mouseMoveTracker) {
      this._mouseMoveTracker.releaseMouseMoves();
      this._mouseMoveTracker = null;
    }
  },
  _getMouseMoveTracker: function _getMouseMoveTracker() {
    return this._mouseMoveTracker || new _domLib.DOMMouseMoveTracker(this._onMove, this._onColumnResizeEnd, document.body);
  },
  _onColumnResizeMouseDown: function _onColumnResizeMouseDown(event) {

    this._mouseMoveTracker = this._getMouseMoveTracker();
    this.isKeyDown = true;
    this._cursorDelta = 0;

    this.props.onColumnResizeStart({
      clientX: event.clientX,
      clientY: event.clientY,
      preventDefault: function preventDefault() {}
    });
  },
  componentWillMount: function componentWillMount() {
    this._columnWidth = this.props.columnWidth;
  },
  componentWillReceiveProps: function componentWillReceiveProps(nextProps) {
    if (this.isKeyDown && nextProps.initialEvent && !this._mouseMoveTracker.isDragging()) {
      this._mouseMoveTracker.captureMouseMoves(nextProps.initialEvent);
    }
    if (nextProps.columnWidth !== this.props.columnWidth) {
      this._columnWidth = nextProps.columnWidth;
    }
  },
  componentWillUnmount: function componentWillUnmount() {
    if (this._mouseMoveTracker) {
      this._mouseMoveTracker.releaseMouseMoves();
      this._mouseMoveTracker = null;
    }
  },
  render: function render() {
    var _props = this.props,
        columnLeft = _props.columnLeft,
        height = _props.height;


    var styles = {
      width: 6,
      left: this._columnWidth + columnLeft - 2,
      height: height
    };

    var classes = (0, _classnames2.default)(this.prefix('column-resize-spanner'));

    return _react2.default.createElement('div', {
      className: classes,
      style: styles,
      onMouseDown: this._onColumnResizeMouseDown,
      ref: 'spanner'
    });
  }
});

exports.default = ColumnResizeHandler;