'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _Cell = require('./Cell');

var _Cell2 = _interopRequireDefault(_Cell);

var _ClassNameMixin = require('./mixins/ClassNameMixin');

var _ClassNameMixin2 = _interopRequireDefault(_ClassNameMixin);

var _ColumnResizeHandler = require('./ColumnResizeHandler');

var _ColumnResizeHandler2 = _interopRequireDefault(_ColumnResizeHandler);

var _ReactComponentWithPureRenderMixin = require('./mixins/ReactComponentWithPureRenderMixin');

var _ReactComponentWithPureRenderMixin2 = _interopRequireDefault(_ReactComponentWithPureRenderMixin);

var _isNullOrUndefined = require('./utils/isNullOrUndefined');

var _isNullOrUndefined2 = _interopRequireDefault(_isNullOrUndefined);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var HeaderCell = _react2.default.createClass({
  displayName: 'HeaderCell',

  mixins: [_ClassNameMixin2.default, _ReactComponentWithPureRenderMixin2.default],
  propTypes: {
    sortable: _react.PropTypes.bool,
    resizable: _react.PropTypes.bool,
    onColumnResizeEnd: _react.PropTypes.func,
    onColumnResizeStart: _react.PropTypes.func,
    onColumnResizeMove: _react.PropTypes.func,
    onSortColumn: _react.PropTypes.func,
    headerHeight: _react.PropTypes.number
  },
  _onColumnResizeStart: function _onColumnResizeStart(event) {
    var _props = this.props,
        left = _props.left,
        fixed = _props.fixed;

    this.setState({ initialEvent: event });
    this.props.onColumnResizeStart(this.state.columnWidth, left, fixed);
  },
  _onColumnResizeEnd: function _onColumnResizeEnd(columnWidth, cursorDelta) {
    this.setState({ columnWidth: columnWidth });
    this.props.onColumnResizeEnd(columnWidth, cursorDelta, this.props.dataKey, this.props.index);
  },
  getInitialState: function getInitialState() {
    var _props2 = this.props,
        width = _props2.width,
        flexGrow = _props2.flexGrow;

    return {
      columnWidth: (0, _isNullOrUndefined2.default)(flexGrow) ? width : 0
    };
  },
  renderResizeSpanner: function renderResizeSpanner() {
    var _props3 = this.props,
        resizable = _props3.resizable,
        left = _props3.left,
        onColumnResizeMove = _props3.onColumnResizeMove,
        fixed = _props3.fixed,
        headerHeight = _props3.headerHeight;
    var _state = this.state,
        columnWidth = _state.columnWidth,
        initialEvent = _state.initialEvent;


    if (!resizable) {
      return null;
    }

    return _react2.default.createElement(_ColumnResizeHandler2.default, {
      columnWidth: columnWidth,
      columnLeft: left,
      columnFixed: fixed,
      height: headerHeight,
      initialEvent: initialEvent,
      onColumnResizeMove: onColumnResizeMove,
      onColumnResizeStart: this._onColumnResizeStart,
      onColumnResizeEnd: this._onColumnResizeEnd
    });
  },
  renderSortColumn: function renderSortColumn() {
    var _props4 = this.props,
        left = _props4.left,
        headerHeight = _props4.headerHeight,
        sortable = _props4.sortable,
        sortColumn = _props4.sortColumn,
        sortType = _props4.sortType,
        dataKey = _props4.dataKey,
        width = _props4.width;
    var columnWidth = this.state.columnWidth;

    var styles = {
      left: (columnWidth || width) + left - 16,
      top: headerHeight / 2 - 8
    };

    if (sortable) {
      return _react2.default.createElement(
        'div',
        {
          style: styles,
          className: this.prefix('sortable')
        },
        _react2.default.createElement('i', { className: sortColumn === dataKey ? 'icon icon-sort-' + sortType : 'icon icon-sort' })
      );
    }

    return null;
  },
  handleClick: function handleClick() {
    var _props5 = this.props,
        sortable = _props5.sortable,
        dataKey = _props5.dataKey,
        sortType = _props5.sortType,
        onSortColumn = _props5.onSortColumn;

    sortable && onSortColumn && onSortColumn(dataKey, sortType === 'asc' ? 'desc' : 'asc');
  },
  render: function render() {

    var classes = this.prefix('cell-header');
    var sortable = this.props.sortable;

    return _react2.default.createElement(
      'div',
      { className: classes },
      _react2.default.createElement(_Cell2.default, _extends({}, this.props, {
        isHeaderCell: true,
        onClick: this.handleClick
      })),
      this.renderSortColumn(),
      this.renderResizeSpanner()
    );
  }
});

exports.default = HeaderCell;