'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _ClassNameMixin = require('./mixins/ClassNameMixin');

var _ClassNameMixin2 = _interopRequireDefault(_ClassNameMixin);

var _domLib = require('dom-lib');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var BAR_MIN_WIDTH = 20;

var Scrollbar = _react2.default.createClass({
  displayName: 'Scrollbar',

  mixins: [_ClassNameMixin2.default],
  propTypes: {
    vertical: _react.PropTypes.bool,
    length: _react.PropTypes.number,
    scrollLength: _react.PropTypes.number,
    onScroll: _react.PropTypes.func
  },
  getInitialState: function getInitialState() {
    return {
      handleDown: false
    };
  },
  getMouseMoveTracker: function getMouseMoveTracker() {
    return this._mouseMoveTracker || new _domLib.DOMMouseMoveTracker(this.hanldeDragMove, this.hanldeDragEnd, document.body);
  },
  releaseMouseMoves: function releaseMouseMoves() {
    if (this._mouseMoveTracker) {
      this._mouseMoveTracker.releaseMouseMoves();
      this._mouseMoveTracker = null;
    }
  },
  hanldeMouseDown: function hanldeMouseDown(event) {

    this._mouseMoveTracker = this.getMouseMoveTracker();
    this._mouseMoveTracker.captureMouseMoves(event);
    this.setState({
      handleDown: true
    });
  },
  hanldeDragEnd: function hanldeDragEnd() {
    this.releaseMouseMoves();
    this.setState({
      handleDown: false
    });
  },
  handleScroll: function handleScroll(delta, event) {
    var _props = this.props,
        length = _props.length,
        scrollLength = _props.scrollLength,
        vertical = _props.vertical,
        onScroll = _props.onScroll;

    var style = {};
    var scrollDelta = delta * (scrollLength / length);
    this.updateScrollBarPosition(delta);
    onScroll && onScroll(scrollDelta, event);
  },
  resetScrollBarPosition: function resetScrollBarPosition() {
    this.scrollOffset = 0;
    this.updateScrollBarPosition(0);
  },
  updateScrollBarPosition: function updateScrollBarPosition(delta) {
    var _props2 = this.props,
        vertical = _props2.vertical,
        length = _props2.length,
        scrollLength = _props2.scrollLength;

    var max = length - length / scrollLength * length;
    var styles = {};

    this.scrollOffset += delta;
    this.scrollOffset = Math.max(this.scrollOffset, 0);
    this.scrollOffset = Math.min(this.scrollOffset, max);

    if (vertical) {
      (0, _domLib.translateDOMPositionXY)(styles, 0, this.scrollOffset);
    } else {
      (0, _domLib.translateDOMPositionXY)(styles, this.scrollOffset, 0);
    }
    (0, _domLib.addStyle)(this.handle, styles);
  },
  onWheelScroll: function onWheelScroll(delta) {
    var _props3 = this.props,
        length = _props3.length,
        scrollLength = _props3.scrollLength;

    var nextDelta = delta / (scrollLength / length);
    this.updateScrollBarPosition(nextDelta);
  },
  hanldeDragMove: function hanldeDragMove(deltaX, deltaY, event) {
    var vertical = this.props.vertical;

    if (!this._mouseMoveTracker || !this._mouseMoveTracker.isDragging()) {
      return;
    }
    this.handleScroll(vertical ? deltaY : deltaX, event);
  },
  componentWillMount: function componentWillMount() {
    this.scrollOffset = 0;
  },
  componentWillUnmount: function componentWillUnmount() {
    this.releaseMouseMoves();
  },
  render: function render() {
    var _styles,
        _this = this;

    var _props4 = this.props,
        vertical = _props4.vertical,
        length = _props4.length,
        scrollLength = _props4.scrollLength;
    var handleDown = this.state.handleDown;


    var classes = (0, _classnames2.default)(this.prefix('scrollbar-wrapper'), {
      vertical: vertical,
      horizontal: !vertical,
      hide: scrollLength <= length,
      active: handleDown
    });

    var styles = (_styles = {}, _defineProperty(_styles, vertical ? 'height' : 'width', length / scrollLength * 100 + '%'), _defineProperty(_styles, vertical ? 'minHeight' : 'minWidth', BAR_MIN_WIDTH), _styles);

    return _react2.default.createElement(
      'div',
      { className: classes },
      _react2.default.createElement('div', {
        ref: function ref(_ref) {
          return _this.handle = _ref;
        },
        className: 'scrollbar-handle',
        style: styles,
        onMouseDown: this.hanldeMouseDown
      })
    );
  }
});

exports.default = Scrollbar;