'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _domLib = require('dom-lib');

var _lodash = require('lodash');

var _Row = require('./Row');

var _Row2 = _interopRequireDefault(_Row);

var _CellGroup = require('./CellGroup');

var _CellGroup2 = _interopRequireDefault(_CellGroup);

var _ClassNameMixin = require('./mixins/ClassNameMixin');

var _ClassNameMixin2 = _interopRequireDefault(_ClassNameMixin);

var _ReactComponentWithPureRenderMixin = require('./mixins/ReactComponentWithPureRenderMixin');

var _ReactComponentWithPureRenderMixin2 = _interopRequireDefault(_ReactComponentWithPureRenderMixin);

var _debounce = require('./utils/debounce');

var _debounce2 = _interopRequireDefault(_debounce);

var _Scrollbar = require('./Scrollbar');

var _Scrollbar2 = _interopRequireDefault(_Scrollbar);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var handleClass = { add: _domLib.addClass, remove: _domLib.removeClass };
var ReactChildren = _react2.default.Children;
var LAYER_WIDTH = 30;

function getTotalByColumns(columns) {
  var totalFlexGrow = 0;
  var totalWidth = 0;

  var count = function count(items) {
    items.forEach(function (column) {
      if (_react2.default.isValidElement(column)) {
        var _column$props = column.props,
            flexGrow = _column$props.flexGrow,
            _column$props$width = _column$props.width,
            width = _column$props$width === undefined ? 0 : _column$props$width;

        totalFlexGrow += flexGrow || 0;
        totalWidth += flexGrow ? 0 : width;
      } else if (_.isArray(column)) {
        count(column);
      }
    });
  };
  count(columns);
  return {
    totalFlexGrow: totalFlexGrow,
    totalWidth: totalWidth
  };
}

var Table = _react2.default.createClass({
  displayName: 'Table',

  mixins: [_ClassNameMixin2.default, _ReactComponentWithPureRenderMixin2.default],
  propTypes: {
    width: _react.PropTypes.number,
    data: _react.PropTypes.array,
    height: _react.PropTypes.number,
    rowHeight: _react.PropTypes.number,
    headerHeight: _react.PropTypes.number,
    onRowClick: _react.PropTypes.func,
    isTree: _react.PropTypes.bool,
    expand: _react.PropTypes.bool,
    locale: _react.PropTypes.object,
    sortColumn: _react.PropTypes.string,
    sortType: _react.PropTypes.oneOf(['desc', 'asc']),
    /**
     * @callback
     * @params: sortColumn dataKey
     * @params: sortType
     */
    onSortColumn: _react.PropTypes.func,
    onRerenderRowHeight: _react.PropTypes.func,
    onTreeToggleOpen: _react.PropTypes.func,
    disabledScroll: _react.PropTypes.bool,
    hover: _react.PropTypes.bool,
    onScroll: _react.PropTypes.func
  },
  getDefaultProps: function getDefaultProps() {
    return {
      height: 200,
      rowHeight: 36,
      sortType: 'asc',
      hover: true,
      locale: {
        emptyMessage: 'No data found'
      }
    };
  },
  getInitialState: function getInitialState() {
    return {
      width: this.props.width,
      columnWidth: 0,
      dataKey: 0,
      shouldFixedColumn: false,
      contentHeight: 0,
      contentWidth: 0
    };
  },
  componentDidMount: function componentDidMount() {
    this._onWindowResizeListener = (0, _domLib.on)(window, 'resize', (0, _debounce2.default)(this.reportTableWidth, 400));
    this.reportTableWidth();
    this.reportTableContextHeight();
  },
  componentDidUpdate: function componentDidUpdate() {
    this.reportTableContextHeight();
    this.reportTableContentWidth();
    this.updatePosition();
  },
  componentWillUnmount: function componentWillUnmount() {
    if (this._onWheelListener) {
      this._onWheelListener.off();
    }
    if (this._onWindowResizeListener) {
      this._onWindowResizeListener.off();
    }
  },
  getFixedCellGroups: function getFixedCellGroups() {
    return (0, _reactDom.findDOMNode)(this.table).querySelectorAll('.' + this.props.classPrefix + '-cell-group.fixed');
  },
  getScrollCellGroups: function getScrollCellGroups() {
    return (0, _reactDom.findDOMNode)(this.table).querySelectorAll('.' + this.props.classPrefix + '-cell-group.scroll');
  },
  onColumnResizeEnd: function onColumnResizeEnd(columnWidth, cursorDelta, dataKey, index) {
    this.setState(_defineProperty({
      isColumnResizing: false
    }, dataKey + '_' + index + '_width', columnWidth));
    (0, _domLib.addStyle)(this.mouseArea, {
      display: 'none'
    });
  },
  onColumnResizeStart: function onColumnResizeStart(width, left, fixed) {
    this.setState({
      isColumnResizing: true
    });
    var mouseAreaLeft = width + left;
    var x = fixed ? mouseAreaLeft : mouseAreaLeft + (this.scrollX || 0);
    var styles = { display: 'block' };
    (0, _domLib.translateDOMPositionXY)(styles, x, 0);
    (0, _domLib.addStyle)(this.mouseArea, styles);
  },
  onColumnResizeMove: function onColumnResizeMove(width, left, fixed) {
    var mouseAreaLeft = width + left;
    var x = fixed ? mouseAreaLeft : mouseAreaLeft + (this.scrollX || 0);
    (0, _domLib.translateDOMPositionXY)(styles, x, 0);
    (0, _domLib.addStyle)(this.mouseArea, styles);
  },
  onTreeToggle: function onTreeToggle(rowKey, rowIndex, rowData) {
    var onTreeToggleOpen = this.props.onTreeToggleOpen;

    var expandIcon = (0, _reactDom.findDOMNode)(this.refs['children_' + rowKey + '_' + rowIndex]);
    var isOpen = (0, _domLib.hasClass)(expandIcon, 'open');

    if (isOpen) {
      (0, _domLib.removeClass)(expandIcon, 'open');
    } else {
      (0, _domLib.addClass)(expandIcon, 'open');
    }

    this.reportTableContextHeight();
    onTreeToggleOpen && onTreeToggleOpen(!isOpen, rowData);
  },
  cloneCell: function cloneCell(Cell, props) {
    return _react2.default.cloneElement(Cell, props, Cell.props.children);
  },
  getCells: function getCells() {
    var _this = this;

    var left = 0; // Cell left margin
    var headerCells = []; // Table header cell
    var bodyCells = []; // Table body cell
    var columns = this.props.children;

    if (!columns) {
      return {
        headerCells: headerCells,
        bodyCells: bodyCells,
        allColumnsWidth: left
      };
    }

    var _state = this.state,
        dataKey = _state.dataKey,
        columnWidth = _state.columnWidth,
        tableWidth = _state.width;
    var _props = this.props,
        sortColumn = _props.sortColumn,
        sortType = _props.sortType,
        onSortColumn = _props.onSortColumn,
        rowHeight = _props.rowHeight,
        headerHeight = _props.headerHeight;

    var _getTotalByColumns = getTotalByColumns(columns),
        totalFlexGrow = _getTotalByColumns.totalFlexGrow,
        totalWidth = _getTotalByColumns.totalWidth;

    ReactChildren.map(columns, function (column, index) {

      if (_react2.default.isValidElement(column)) {

        var columnChildren = column.props.children;
        var _column$props2 = column.props,
            width = _column$props2.width,
            fixed = _column$props2.fixed,
            align = _column$props2.align,
            sortable = _column$props2.sortable,
            resizable = _column$props2.resizable,
            flexGrow = _column$props2.flexGrow,
            minWidth = _column$props2.minWidth;


        if (columnChildren.length !== 2) {
          throw new Error('Component <HeaderCell> and <Cell> is required, column index: ' + index + ' ');
        }

        var nextWidth = _this.state[columnChildren[1].props.dataKey + '_' + index + '_width'] || width || 0;

        if (tableWidth && flexGrow && totalFlexGrow) {
          nextWidth = Math.max((tableWidth - totalWidth) / totalFlexGrow * flexGrow, minWidth || 60);
        }

        var cellProps = {
          fixed: fixed,
          left: left,
          align: align,
          resizable: resizable,
          sortable: sortable,
          index: index,
          width: nextWidth,
          height: rowHeight,
          headerHeight: headerHeight,
          firstColumn: index === 0,
          lastColumn: index === columns.length - 1,
          flexGrow: flexGrow,
          key: index
        };

        var headerCellsProps = {
          headerHeight: headerHeight || rowHeight,
          dataKey: columnChildren[1].props.dataKey,
          sortColumn: sortColumn,
          sortType: sortType,
          onSortColumn: onSortColumn,
          flexGrow: flexGrow
        };

        if (resizable) {
          headerCellsProps.onColumnResizeEnd = _this.onColumnResizeEnd;
          headerCellsProps.onColumnResizeStart = _this.onColumnResizeStart;
          headerCellsProps.onColumnResizeMove = _this.onColumnResizeMove;
        }

        headerCells.push(_this.cloneCell(columnChildren[0], (0, _lodash.assign)(cellProps, headerCellsProps)));
        bodyCells.push(_this.cloneCell(columnChildren[1], cellProps));

        left += nextWidth;
      }
    });

    return {
      headerCells: headerCells,
      bodyCells: bodyCells,
      allColumnsWidth: left
    };
  },
  renderRow: function renderRow(props, cells) {

    //IF there are fixed columns, add a fixed group
    if (this.state.shouldFixedColumn) {

      var fixedCells = cells.filter(function (cell) {
        return cell.props.fixed;
      });

      var otherCells = cells.filter(function (cell) {
        return !cell.props.fixed;
      });

      var fixedCellGroupWidth = 0;

      fixedCells.map(function (item) {
        fixedCellGroupWidth += item.props.width;
      });

      return _react2.default.createElement(
        _Row2.default,
        props,
        _react2.default.createElement(
          _CellGroup2.default,
          {
            fixed: true,
            height: props.isHeaderRow ? props.headerHeight : props.height,
            width: fixedCellGroupWidth },
          fixedCells
        ),
        _react2.default.createElement(
          _CellGroup2.default,
          null,
          otherCells
        )
      );
    }

    return _react2.default.createElement(
      _Row2.default,
      props,
      _react2.default.createElement(
        _CellGroup2.default,
        null,
        cells
      )
    );
  },
  renderTableHeader: function renderTableHeader(headerCells, rowWidth) {
    var _this2 = this;

    var _props2 = this.props,
        rowHeight = _props2.rowHeight,
        headerHeight = _props2.headerHeight;

    var row = this.renderRow({
      ref: function ref(_ref) {
        return _this2.tableHeader = _ref;
      },
      width: rowWidth,
      height: rowHeight,
      headerHeight: headerHeight,
      isHeaderRow: true,
      top: 0
    }, headerCells);

    return _react2.default.createElement(
      'div',
      {
        ref: function ref(_ref2) {
          return _this2.headerWrapper = _ref2;
        },
        className: this.prefix('header-row-wrapper') },
      row
    );
  },
  randerRowData: function randerRowData(bodyCells, rowData, props) {
    var _this3 = this;

    var onRowClick = this.props.onRowClick;

    var hasChildren = this.props.isTree && rowData.children && Array.isArray(rowData.children) && rowData.children.length > 0;
    var rowKey = '_' + (Math.random() * 1E18).toString(36).slice(0, 5).toUpperCase();

    var row = this.renderRow({
      key: props.index,
      rowIndex: props.index,
      width: props.rowWidth,
      height: props.rowHeight,
      top: props.top,
      onClick: function onClick() {
        onRowClick && onRowClick(rowData);
      },
      rowData: rowData
    }, bodyCells.map(function (cell, key) {
      return _react2.default.cloneElement(cell, {
        key: key,
        layer: props.layer,
        hasChildren: hasChildren,
        height: props.rowHeight,
        rowIndex: props.index,
        onTreeToggle: _this3.onTreeToggle,
        rowKey: rowKey,
        rowData: rowData
      }, cell.props.children);
    }));

    //insert children
    if (hasChildren) {
      props.layer++;

      var childrenClasses = (0, _classnames2.default)(this.prefix('row-children'), {
        open: this.props.expand
      });

      var childrenStyles = {
        marginLeft: LAYER_WIDTH
      };
      return _react2.default.createElement(
        'div',
        { className: childrenClasses,
          'data-layer': props.layer,
          ref: 'children_' + rowKey + '_' + props.index,
          key: props.index },
        row,
        _react2.default.createElement(
          'div',
          { className: 'children' },
          rowData.children.map(function (child, index) {
            return _this3.randerRowData(bodyCells, child, Object.assign({}, props, { index: index }));
          })
        )
      );
    }

    return row;
  },
  renderTableBody: function renderTableBody(bodyCells, rowWidth, allColumnsWidth) {
    var _this4 = this;

    var _props3 = this.props,
        headerHeight = _props3.headerHeight,
        rowHeight = _props3.rowHeight,
        height = _props3.height,
        data = _props3.data,
        isTree = _props3.isTree,
        onRerenderRowHeight = _props3.onRerenderRowHeight,
        disabledScroll = _props3.disabledScroll;


    var bodyStyles = {
      top: isTree ? 0 : headerHeight || rowHeight,
      height: height - (headerHeight || rowHeight)
    };

    var top = 0; //Row position
    var layer = 0; //Tree layer
    var rows = data && data.length > 0 ? data.map(function (rowData, index) {

      var nextRowHeight = rowHeight;

      /**
       * 自定义行高
       */
      if (onRerenderRowHeight) {
        nextRowHeight = onRerenderRowHeight(rowData) || rowHeight;
      }

      var row = _this4.randerRowData(bodyCells, rowData, {
        index: index,
        top: top,
        rowWidth: rowWidth,
        layer: layer,
        rowHeight: nextRowHeight
      });

      !isTree && (top += nextRowHeight);

      return row;
    }) : null;

    var wheelStyles = {
      position: 'absolute'
    };

    return _react2.default.createElement(
      'div',
      { ref: function ref(_ref6) {
          return _this4.tableBody = _ref6;
        },
        className: this.prefix('body-row-wrapper'),
        style: bodyStyles,
        onWheel: this.wheelHandler.onWheel
      },
      _react2.default.createElement(
        'div',
        {
          style: wheelStyles,
          className: this.prefix('body-wheel-area'),
          ref: function ref(_ref3) {
            return _this4.wheelWrapper = _ref3;
          }
        },
        rows
      ),
      rows ? null : _react2.default.createElement(
        'div',
        { className: this.prefix('body-info') },
        this.props.locale.emptyMessage
      ),
      disabledScroll ? null : _react2.default.createElement(
        'div',
        null,
        _react2.default.createElement(_Scrollbar2.default, {
          length: this.state.width,
          onScroll: this.handleScrollX,
          scrollLength: this.state.contentWidth,
          ref: function ref(_ref4) {
            return _this4.scrollbarX = _ref4;
          }
        }),
        _react2.default.createElement(_Scrollbar2.default, {
          vertical: true,
          length: height - (headerHeight || rowHeight),
          scrollLength: this.state.contentHeight,
          onScroll: this.handleScrollY,
          ref: function ref(_ref5) {
            return _this4.scrollbarY = _ref5;
          }
        })
      )
    );
  },
  renderMouseArea: function renderMouseArea() {
    var _this5 = this;

    var height = this.props.height;

    var styles = { height: height };

    return _react2.default.createElement('div', {
      ref: function ref(_ref7) {
        return _this5.mouseArea = _ref7;
      },
      className: this.prefix('mouse-area'),
      style: styles
    });
  },
  handleScrollX: function handleScrollX(delta, event) {
    this.handleWheel(delta, 0);
  },
  handleScrollY: function handleScrollY(delta, event) {
    this.handleWheel(0, delta);
  },
  handleWheel: function handleWheel(deltaX, deltaY) {
    var onScroll = this.props.onScroll;


    if (!this.isMounted()) {
      return;
    }

    var nextScrollX = this.scrollX - deltaX;
    var nextScrollY = this.scrollY - deltaY;

    this.scrollY = Math.min(0, nextScrollY < this.minScrollY ? this.minScrollY : nextScrollY);
    this.scrollX = Math.min(0, nextScrollX < this.minScrollX ? this.minScrollX : nextScrollX);
    this.updatePosition();

    onScroll && onScroll(this.scrollX, this.scrollY);
  },
  updatePosition: function updatePosition() {
    /**
     * 当存在锁定列情况处理
     */
    if (this.state.shouldFixedColumn) {
      this.updatePositionByFixedCell();
    } else {
      var wheelStyle = {};
      var headerStyle = {};
      (0, _domLib.translateDOMPositionXY)(wheelStyle, this.scrollX, this.scrollY);
      (0, _domLib.translateDOMPositionXY)(headerStyle, this.scrollX, 0);
      this.wheelWrapper && (0, _domLib.addStyle)(this.wheelWrapper, wheelStyle);
      this.headerWrapper && (0, _domLib.addStyle)(this.headerWrapper, headerStyle);
    }
    handleClass[this.scrollY < 0 ? 'add' : 'remove']((0, _reactDom.findDOMNode)(this.tableHeader), 'shadow');
  },
  updatePositionByFixedCell: function updatePositionByFixedCell() {
    var _this6 = this;

    var wheelGroupStyle = {};
    var wheelStyle = {};
    var scrollGroups = this.getScrollCellGroups();
    var fixedGroups = this.getFixedCellGroups();

    (0, _domLib.translateDOMPositionXY)(wheelGroupStyle, this.scrollX, 0);
    (0, _domLib.translateDOMPositionXY)(wheelStyle, 0, this.scrollY);

    Array.from(scrollGroups).map(function (group) {
      (0, _domLib.addStyle)(group, wheelGroupStyle);
    });

    (0, _domLib.addStyle)(this.wheelWrapper, wheelStyle);
    Array.from(fixedGroups).map(function (group) {
      handleClass[_this6.scrollX < 0 ? 'add' : 'remove'](group, 'shadow');
    });
  },
  shouldHandleWheelX: function shouldHandleWheelX(delta) {

    if (delta === 0 || this.props.disabledScroll) {
      return false;
    }

    return delta >= 0 && this.scrollX > this.minScrollX || delta < 0 && this.scrollX < 0;
  },
  shouldHandleWheelY: function shouldHandleWheelY(delta) {

    if (delta === 0 || this.props.disabledScroll) {
      return false;
    }
    return delta >= 0 && this.scrollY > this.minScrollY || delta < 0 && this.scrollY < 0;
  },
  componentWillMount: function componentWillMount() {
    var _this7 = this;

    var children = this.props.children;


    if (!children) {
      return;
    }
    var shouldFixedColumn = children.some(function (child) {
      return child.props && child.props.fixed;
    });

    this.scrollY = 0;
    this.scrollX = 0;
    this.wheelHandler = new _domLib.WheelHandler(function (deltaX, deltaY) {
      _this7.handleWheel(deltaX, deltaY);
      _this7.scrollbarX.onWheelScroll(deltaX);
      _this7.scrollbarY.onWheelScroll(deltaY);
    }, this.shouldHandleWheelX, this.shouldHandleWheelY);
    this.setState({ shouldFixedColumn: shouldFixedColumn });
  },
  reportTableWidth: function reportTableWidth() {
    this.setState({
      width: (0, _domLib.getWidth)((0, _reactDom.findDOMNode)(this.table))
    });
  },
  reportTableContentWidth: function reportTableContentWidth() {
    var row = (0, _reactDom.findDOMNode)(this.table).querySelectorAll('.' + this.props.classPrefix + '-row-header')[0];
    var contentWidth = (0, _domLib.getWidth)(row);

    this.setState({ contentWidth: contentWidth });
    // 这里 -10 是为了让滚动条不挡住内容部分
    this.minScrollX = -(contentWidth - this.state.width) - 10;

    if (this.state.contentWidth !== contentWidth) {
      this.scrollX = 0;
      this.scrollbarX && this.scrollbarX.resetScrollBarPosition();
    }
  },
  reportTableContextHeight: function reportTableContextHeight() {

    var rows = (0, _reactDom.findDOMNode)(this.table).querySelectorAll('.' + this.props.classPrefix + '-row');
    var _props4 = this.props,
        height = _props4.height,
        rowHeight = _props4.rowHeight,
        headerHeight = _props4.headerHeight;

    var contentHeight = 0;
    Array.from(rows).forEach(function (row, index) {
      contentHeight += (0, _domLib.getHeight)(row);
    });

    var nextContentHeight = contentHeight - (headerHeight || rowHeight);
    this.setState({
      contentHeight: nextContentHeight
    });
    // 这里 -10 是为了让滚动条不挡住内容部分
    this.minScrollY = -(contentHeight - height) - 10;
    if (this.state.contentHeight !== nextContentHeight) {
      this.scrollY = 0;
      this.scrollbarY && this.scrollbarY.resetScrollBarPosition();
    }
  },
  render: function render() {
    var _this8 = this;

    var _props5 = this.props,
        children = _props5.children,
        className = _props5.className,
        _props5$width = _props5.width,
        width = _props5$width === undefined ? 0 : _props5$width,
        height = _props5.height,
        style = _props5.style,
        rowHeight = _props5.rowHeight,
        classPrefix = _props5.classPrefix,
        isTree = _props5.isTree,
        hover = _props5.hover,
        id = _props5.id;

    var _getCells = this.getCells(),
        headerCells = _getCells.headerCells,
        bodyCells = _getCells.bodyCells,
        allColumnsWidth = _getCells.allColumnsWidth;

    var rowWidth = allColumnsWidth > width ? allColumnsWidth : width;

    var clesses = (0, _classnames2.default)(classPrefix, isTree ? this.prefix('treetable') : '', className, {
      'column-resizing': this.state.isColumnResizing,
      'table-hover': hover
    });

    var styles = (0, _lodash.assign)({ width: width || 'auto', height: height }, style);

    return _react2.default.createElement(
      'div',
      { className: clesses, style: styles, ref: function ref(_ref8) {
          return _this8.table = _ref8;
        }, id: id },
      this.renderTableHeader(headerCells, rowWidth),
      children && this.renderTableBody(bodyCells, rowWidth, allColumnsWidth),
      this.renderMouseArea()
    );
  }
});

exports.default = Table;